@@ -126,6 +126,8 @@ gcpDocument::~gcpDocument ()
 	pango_attr_list_unref (m_PangoAttrList);
 	if (m_Theme)
 		m_Theme->RemoveClient (this);
+	if (m_App)
+		static_cast<gcpApplication*> (m_App)->SetActiveDocument (NULL);
 }
 
 GtkWidget* gcpDocument::GetWidget()
