@@ -30,10 +30,10 @@
 #include "tool.h"
 #include "stringdlg.h"
 #include <glib/gi18n-lib.h>
-#include <math.h>
 #include <unistd.h>
-#include <locale.h>
 #include <openbabel/obconversion.h>
+#include <cmath>
+#include <clocale>
 
 static void do_export_to_ghemical (gcpMolecule* pMol)
 {
@@ -352,7 +352,7 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		SetId (buf);
 		xmlFree (buf);
 	}
-	child = GetNodeByName (node, "atom");
+	child = GetNodeByName (node, (char*) "atom");
 	while (child) {
 		pObject = new gcpAtom ();
 		if (pDoc)
@@ -364,10 +364,10 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		if (pDoc)
 			pDoc->AddAtom ((gcpAtom*) pObject);
 		AddAtom ((gcpAtom*) pObject);
-		child = GetNextNodeByName (child->next, "atom");
+		child = GetNextNodeByName (child->next, (char*) "atom");
 	}
 	
-	child = GetNodeByName (node, "fragment");
+	child = GetNodeByName (node, (char*) "fragment");
 	while (child) {
 		pObject = new gcpFragment ();
 		AddChild (pObject);
@@ -378,10 +378,10 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		if (pDoc)
 			pDoc->AddFragment ((gcpFragment*) pObject);
 		AddFragment ((gcpFragment*) pObject);
-		child = GetNextNodeByName (child->next, "fragment");
+		child = GetNextNodeByName (child->next, (char*) "fragment");
 	}
 
-	child = GetNodeByName (node, "bond");
+	child = GetNodeByName (node, (char*) "bond");
 	while (child) {
 		pObject = new gcpBond ();
 		AddBond ((gcpBond*) pObject);
@@ -392,7 +392,7 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		}
 		if (pDoc)
 			pDoc->AddBond ((gcpBond*) pObject);
-		child = GetNextNodeByName (child->next, "bond");
+		child = GetNextNodeByName (child->next, (char*) "bond");
 		CheckCrossings ((gcpBond*) pObject);
 	}
 	if (!m_Atoms.empty ()) {
@@ -498,7 +498,7 @@ double gcpMolecule::GetYAlign ()
 	return (miny + maxy) / 2.0;
 }
 
-bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
+bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object, double x, double y)
 {
 	bool result = false;
 	GtkActionGroup *group = gtk_action_group_new ("molecule");
@@ -510,29 +510,35 @@ bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 			action = gtk_action_new ("ghemical", _("Export molecule to Ghemical"), NULL, NULL);
 			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_export_to_ghemical), this);
 			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='ghemical'/></menu></popup></ui>", -1, NULL);
 		}
 		if (((gcpDocument*) GetDocument ())->GetApplication ()->HaveInChI ()) {
 			action = gtk_action_new ("inchi", _("Generate InChI"), NULL, NULL);
 			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_inchi), this);
 			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='inchi'/></menu></popup></ui>", -1, NULL);
 			action = gtk_action_new ("webbook", _("NIST WebBook page for this molecule"), NULL, NULL);
 			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_webbook), this);
 			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='webbook'/></menu></popup></ui>", -1, NULL);
 			action = gtk_action_new ("pubchem", _("PubChem page for this molecule"), NULL, NULL);
 			g_signal_connect_swapped (action, "activate", G_CALLBACK (do_show_pubchem), this);
 			gtk_action_group_add_action (group, action);
+			g_object_unref (action);
 			gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='pubchem'/></menu></popup></ui>", -1, NULL);
 		}
 		action = gtk_action_new ("smiles", _("Generate Smiles"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_smiles), this);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='smiles'/></menu></popup></ui>", -1, NULL);
 		action = gtk_action_new ("calc", _("Open in Calculator"), NULL, NULL);
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_open_in_calc), this);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='calc'/></menu></popup></ui>", -1, NULL);
 		result = true;
 	}
@@ -541,14 +547,14 @@ bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 		g_signal_connect (action, "activate", G_CALLBACK (do_select_alignment), this);
 		g_object_set_data (G_OBJECT (action), "item", object);
 		gtk_action_group_add_action (group, action);
+		g_object_unref (action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='select-align'/></menu></popup></ui>", -1, NULL);
 		result = true;
 	}
 	if (result)
 		gtk_ui_manager_insert_action_group (UIManager, group, 0);
-	else
-		g_object_unref (group);
-	return result | GetParent ()->BuildContextualMenu (UIManager, object);
+	g_object_unref (group);
+	return result | GetParent ()->BuildContextualMenu (UIManager, object, x, y);
 }
 
 void gcpMolecule::ExportToGhemical ()
@@ -807,8 +813,7 @@ void gcpMolecule::OpenCalc ()
 	list<gcpAtom*>::iterator ia, enda = m_Atoms.end ();
 	ostringstream ofs;
 	int nH;
-#warning "the following line should be edited for stable releases"
-	ofs << "gchemcalc-unstable ";
+	ofs << "gchemcalc ";
 	for (ia = m_Atoms.begin(); ia != enda; ia++) {
 		ofs << (*ia)->GetSymbol();
 		nH = (*ia)->GetAttachedHydrogens ();
